@@ -14,7 +14,7 @@ import fontawesome.icons as fontawesome
 from pkg_resources import resource_filename
 from si_prefix import si_format
 
-__version__ = '0.1.0'
+__version__ = '0.1.2'
 
 # we don't run a comprehensive test suite and mostly in notebooks,
 # so show warnings all the time.
@@ -207,6 +207,6 @@ def infographic(items, **kwargs):
                 x=0.65, y=0.4, text=fontawesome[icon], text_color="#6BAED6",
                 text_font="FontAwesome", text_font_size="48px"))
         plots.append(p)
-    defaults = {'toolbar_location': None}
+    defaults = {'toolbar_location': None, "ncols": len(items)}
     defaults.update(kwargs)
     return gridplot(plots, **defaults)
